@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,14 +21,18 @@
 #define cppcheckH
 //---------------------------------------------------------------------------
 
-#include "config.h"
-#include "settings.h"
-#include "errorlogger.h"
+#include "analyzerinfo.h"
 #include "check.h"
+#include "config.h"
+#include "errorlogger.h"
+#include "importproject.h"
+#include "settings.h"
 
-#include <string>
-#include <list>
+#include <cstddef>
 #include <istream>
+#include <list>
+#include <map>
+#include <string>
 
 class Tokenizer;
 
@@ -68,6 +72,7 @@ public:
       *  settings()).
       */
     unsigned int check(const std::string &path);
+    unsigned int check(const ImportProject::FileSettings &fs);
 
     /**
       * @brief Check the file.
@@ -124,12 +129,19 @@ public:
         _simplify = false;
     }
 
-    /** analyse whole program, run this after all TUs has been scanned. */
-    void analyseWholeProgram();
+    /** Analyse whole program, run this after all TUs has been scanned.
+     * This is deprecated and the plan is to remove this when
+     * .analyzeinfo is good enough.
+     * Return true if an error is reported.
+     */
+    bool analyseWholeProgram();
+
+    /** analyse whole program use .analyzeinfo files */
+    void analyseWholeProgram(const std::string &buildDir, const std::map<std::string, std::size_t> &files);
 
     /** Check if the user wants to check for unused functions
      * and if it's possible at all */
-    bool unusedFunctionCheckIsEnabled() const;
+    bool isUnusedFunctionCheckEnabled() const;
 
 private:
 
@@ -137,28 +149,29 @@ private:
     void internalError(const std::string &filename, const std::string &msg);
 
     /**
-     * @brief Process one file.
+     * @brief Check a file using stream
      * @param filename file name
+     * @param cfgname  cfg name
      * @param fileStream stream the file content can be read from
-     * @return amount of errors found
+     * @return number of errors found
      */
-    unsigned int processFile(const std::string& filename, std::istream& fileStream);
+    unsigned int checkFile(const std::string& filename, const std::string &cfgname, std::istream& fileStream);
 
     /**
      * @brief Check raw tokens
-     * @param tokenizer
+     * @param tokenizer tokenizer instance
      */
     void checkRawTokens(const Tokenizer &tokenizer);
 
     /**
      * @brief Check normal tokens
-     * @param tokenizer
+     * @param tokenizer tokenizer instance
      */
     void checkNormalTokens(const Tokenizer &tokenizer);
 
     /**
      * @brief Check simplified tokens
-     * @param tokenizer
+     * @param tokenizer tokenizer instance
      */
     void checkSimplifiedTokens(const Tokenizer &tokenizer);
 
@@ -176,24 +189,24 @@ private:
      * "[filepath:line number] Message", e.g.
      * "[main.cpp:4] Uninitialized member variable"
      */
-    virtual void reportErr(const ErrorLogger::ErrorMessage &msg);
+    virtual void reportErr(const ErrorLogger::ErrorMessage &msg) override;
 
     /**
      * @brief Information about progress is directed here.
      *
      * @param outmsg Message to show, e.g. "Checking main.cpp..."
      */
-    virtual void reportOut(const std::string &outmsg);
+    virtual void reportOut(const std::string &outmsg) override;
 
     std::list<std::string> _errorList;
     Settings _settings;
 
-    void reportProgress(const std::string &filename, const char stage[], const std::size_t value);
+    void reportProgress(const std::string &filename, const char stage[], const std::size_t value) override;
 
     /**
      * Output information messages.
      */
-    virtual void reportInfo(const ErrorLogger::ErrorMessage &msg);
+    virtual void reportInfo(const ErrorLogger::ErrorMessage &msg) override;
 
     ErrorLogger &_errorLogger;
 
@@ -212,6 +225,8 @@ private:
 
     /** File info used for whole program analysis */
     std::list<Check::FileInfo*> fileInfo;
+
+    AnalyzerInformation analyzerInformation;
 };
 
 /// @}
