@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,12 @@
 #include "config.h"
 #include "suppressions.h"
 
+#include <cstddef>
+#include <fstream>
 #include <list>
 #include <string>
+#include <utility>
+#include <vector>
 
 /**
  * CWE id (Common Weakness Enumeration)
@@ -41,12 +45,16 @@ struct CWE {
 class Token;
 class TokenList;
 
+namespace tinyxml2 {
+    class XMLElement;
+}
+
 /// @addtogroup Core
 /// @{
 
 /** @brief Simple container to be thrown when internal error is detected. */
 struct InternalError {
-    enum Type {SYNTAX, INTERNAL};
+    enum Type {AST, SYNTAX, INTERNAL};
     InternalError(const Token *tok, const std::string &errorMsg, Type type = INTERNAL);
     const Token *token;
     std::string errorMessage;
@@ -155,11 +163,17 @@ public:
     }
 };
 
+
+typedef std::pair<const Token *, std::string> ErrorPathItem;
+typedef std::list<ErrorPathItem> ErrorPath;
+
 /**
  * @brief This is an interface, which the class responsible of error logging
  * should implement.
  */
 class CPPCHECKLIB ErrorLogger {
+protected:
+    std::ofstream plistFile;
 public:
 
     /**
@@ -175,14 +189,19 @@ public:
         class CPPCHECKLIB FileLocation {
         public:
             FileLocation()
-                : line(0) {
+                : fileIndex(0), line(0), col(0) {
             }
 
             FileLocation(const std::string &file, unsigned int aline)
-                : line(aline), _file(file) {
+                : fileIndex(0), line(aline), col(0), _file(file) {
             }
 
-            FileLocation(const Token* tok, const TokenList* list);
+            FileLocation(const std::string &file, const std::string &info, unsigned int aline)
+                : fileIndex(0), line(aline), col(0), _file(file), _info(info) {
+            }
+
+            FileLocation(const Token* tok, const TokenList* tokenList);
+            FileLocation(const Token* tok, const std::string &info, const TokenList* tokenList);
 
             /**
              * Return the filename.
@@ -202,36 +221,48 @@ public:
              */
             std::string stringify() const;
 
+            unsigned int fileIndex;
             unsigned int line;
+            unsigned int col;
+
+            std::string getinfo() const {
+                return _info;
+            }
+            void setinfo(const std::string &i) {
+                _info = i;
+            }
 
         private:
             std::string _file;
+            std::string _info;
         };
 
-        ErrorMessage(const std::list<FileLocation> &callStack, Severity::SeverityType severity, const std::string &msg, const std::string &id, bool inconclusive);
-        ErrorMessage(const std::list<FileLocation> &callStack, Severity::SeverityType severity, const std::string &msg, const std::string &id, const CWE &cwe, bool inconclusive);
+        ErrorMessage(const std::list<FileLocation> &callStack, const std::string& file1, Severity::SeverityType severity, const std::string &msg, const std::string &id, bool inconclusive);
+        ErrorMessage(const std::list<FileLocation> &callStack, const std::string& file1, Severity::SeverityType severity, const std::string &msg, const std::string &id, const CWE &cwe, bool inconclusive);
         ErrorMessage(const std::list<const Token*>& callstack, const TokenList* list, Severity::SeverityType severity, const std::string& id, const std::string& msg, bool inconclusive);
         ErrorMessage(const std::list<const Token*>& callstack, const TokenList* list, Severity::SeverityType severity, const std::string& id, const std::string& msg, const CWE &cwe, bool inconclusive);
+        ErrorMessage(const ErrorPath &errorPath, const TokenList *tokenList, Severity::SeverityType severity, const char id[], const std::string &msg, const CWE &cwe, bool inconclusive);
         ErrorMessage();
+        explicit ErrorMessage(const tinyxml2::XMLElement * const errmsg);
 
         /**
          * Format the error message in XML format
-         * @param verbose use verbose message
-         * @param ver XML version
          */
-        std::string toXML(bool verbose, int ver) const;
+        std::string toXML() const;
 
-        static std::string getXMLHeader(int xml_version);
-        static std::string getXMLFooter(int xml_version);
+        static std::string getXMLHeader();
+        static std::string getXMLFooter();
 
         /**
          * Format the error message into a string.
          * @param verbose use verbose message
-         * @param outputFormat Empty string to use default output format
+         * @param templateFormat Empty string to use default output format
          * or template to be used. E.g. "{file}:{line},{severity},{id},{message}"
+         * @param templateLocation Format Empty string to use default output format
+         * or template to be used. E.g. "{file}:{line},{info}"
         * @return formatted string
          */
-        std::string toString(bool verbose, const std::string &outputFormat = emptyString) const;
+        std::string toString(bool verbose, const std::string &templateFormat = emptyString, const std::string &templateLocation = emptyString) const;
 
         std::string serialize() const;
         bool deserialize(const std::string &data);
@@ -259,6 +290,13 @@ public:
             return _verboseMessage;
         }
 
+        /** Symbol names */
+        const std::string &symbolNames() const {
+            return _symbolNames;
+        }
+
+        Suppressions::ErrorMessage toSuppressionsErrorMessage() const;
+
     private:
         /**
          * Replace all occurrences of searchFor with replaceWith in the
@@ -276,10 +314,18 @@ public:
 
         /** Verbose message */
         std::string _verboseMessage;
+
+        /** symbol names */
+        std::string _symbolNames;
     };
 
     ErrorLogger() { }
-    virtual ~ErrorLogger() { }
+    virtual ~ErrorLogger() {
+        if (plistFile.is_open()) {
+            plistFile << ErrorLogger::plistFooter();
+            plistFile.close();
+        }
+    }
 
     /**
      * Information about progress is directed here.
@@ -321,16 +367,24 @@ public:
      * Report list of unmatched suppressions
      * @param unmatched list of unmatched suppressions (from Settings::Suppressions::getUnmatched(Local|Global)Suppressions)
      */
-    void reportUnmatchedSuppressions(const std::list<Suppressions::SuppressionEntry> &unmatched);
+    void reportUnmatchedSuppressions(const std::list<Suppressions::Suppression> &unmatched);
 
     static std::string callStackToString(const std::list<ErrorLogger::ErrorMessage::FileLocation> &callStack);
 
     /**
      * Convert XML-sensitive characters into XML entities
      * @param str The input string containing XML-sensitive characters
-     * @return The ouput string containing XML entities
+     * @return The output string containing XML entities
      */
     static std::string toxml(const std::string &str);
+
+    static std::string plistHeader(const std::string &version, const std::vector<std::string> &files);
+    static std::string plistData(const ErrorLogger::ErrorMessage &msg);
+    static const char *plistFooter() {
+        return " </array>\r\n"
+               "</dict>\r\n"
+               "</plist>";
+    }
 };
 
 /// @}
